from setuptools import setup

setup(
    name='fin-tools',
    packages=['fin_tools'],
    version='0.1.05',
    description='common financial tools for risk and pricing',
    url='https://github.com/laholmes/fin-tools',
    author='laurence holmes',
    author_email='laholmes@hotmail.co.uk'
)
